import type { ISubscription } from '@matterport/webcomponent';
export declare type ChangeObserver<T = never> = (newValue: T) => void;
/**
 * Observable: Let something subscribe to changes to something else
 *
 * Abstract base class for all the other Observable* classes. Implements
 * the basic onChange(), notification and dirty concepts.
 */
export declare class Observable<T> {
    readonly isObservable: boolean;
    /**
     * Does this Observable have gossip to share with its observers?
     */
    _dirtyObservable: boolean;
    /**
     * If an Observable is active, it will immediately notify any listeners
     * about its changes. It can be useful to set this to false while doing many changes together
     */
    private _activeObservable;
    /**
     * Callbacks for when any property or element of this instance of Observable change
     */
    private _changeObservers;
    /**
     * Observables which this Observable will propagate its changes to
     */
    private _parentObservables;
    private _childObservables;
    constructor();
    /**
     * Register a callback to be called whenever any kind of change occurs with this object.
     * It is up to the deriving classes to determine what a "change" is.
     *
     * @param {ChangeObserver<T>} [observer]
     * @memberof Observable
     */
    onChanged(observer: ChangeObserver<T>): ISubscription;
    /**
     * Remove a specific callback registered to respond to any change event.
     *
     * @param {ChangeObserver} [observer]
     * @memberof Observable
     */
    removeOnChanged(observer: ChangeObserver<T>): void;
    /**
     * Call the "anything changed" callbacks associated with 'observable'.
     * Also calls it for any ancestors, then finally clears the dirty flag.
     */
    protected notifyObservers(): void;
    protected notifyObserver(observer: ChangeObserver<T>): void;
    /**
     * Set this object and all Observable ancestors as dirty,
     * then notify all observers. Generally used internally by the Observable* implementations,
     * but can also be triggered manually if needed
     */
    setDirty(): void;
    private setDirtyRecursive;
    /**
     * Add an Observable as a child to this one (if it's Observable)
     * This is what ensures onChange() notifications bubble up correctly
     * and setActive() flows down correctly
     */
    protected addChildObservable(obj: T | Observable<any>): void;
    /**
     * Remove an Observable as a child to this one (if it's Observable)
     * This is what ensures onChange() notifications bubble up correctly,
     * and setActive() flows down correctly
     */
    protected removeChildObservables(obj: T | Observable<any>): void;
    /**
     * Run a function *atomically*, meaning that even if you
     * mutate lots of parts of the Observable dozens of times,
     * listeners will only be notified once
     * @param func to fun
     */
    atomic(func: () => void): void;
    /**
     * Set whether this Observable (and all descendants) should be
     * considered active, aka notifying about changes
     * @param value
     */
    protected setActive(value: boolean): void;
    /**
     * Return a deep copy of the target object of the observable,
     * without any observable members. See also the deepCopy util function
     */
    deepCopy(): any;
    /**
     * Convenient static function to type check if object is Observable
     * @param obj
     */
    static isObservable<T>(obj: any): obj is Observable<T>;
}
