export class Observable {
    constructor() {
        this.isObservable = true;
        this._dirtyObservable = false;
        this._activeObservable = true;
        this._changeObservers = new Set();
        this._parentObservables = new Set();
        this._childObservables = new Set();
    }
    onChanged(observer) {
        if (this._changeObservers.has(observer)) {
            throw new Error('This observer function is already observing this Observable, ' +
                'and double subscriptions are not supported.');
        }
        const renew = () => this._changeObservers.add(observer);
        const cancel = () => this.removeOnChanged(observer);
        renew();
        return {
            renew,
            cancel,
        };
    }
    removeOnChanged(observer) {
        this._changeObservers.delete(observer);
    }
    notifyObservers() {
        if (this._activeObservable && this._dirtyObservable) {
            for (const changeObserver of this._changeObservers) {
                this.notifyObserver(changeObserver);
            }
            for (const parent of this._parentObservables) {
                parent.notifyObservers();
            }
            this._dirtyObservable = false;
        }
    }
    notifyObserver(observer) {
        observer(undefined);
    }
    setDirty() {
        this.setDirtyRecursive();
        this.notifyObservers();
    }
    setDirtyRecursive() {
        this._dirtyObservable = true;
        for (const parent of this._parentObservables) {
            parent.setDirtyRecursive();
        }
    }
    addChildObservable(obj) {
        if (Observable.isObservable(obj)) {
            this._childObservables.add(obj);
            obj._parentObservables.add(this);
        }
    }
    removeChildObservables(obj) {
        if (Observable.isObservable(obj)) {
            this._childObservables.delete(obj);
            obj._parentObservables.delete(this);
        }
    }
    atomic(func) {
        if (!this._activeObservable) {
            return func();
        }
        try {
            this.setActive(false);
            func();
        }
        finally {
            this.setActive(true);
        }
    }
    setActive(value) {
        const oldValue = this._activeObservable;
        this._activeObservable = value;
        if (!oldValue && value) {
            this.notifyObservers();
        }
        for (const child of this._childObservables) {
            child.setActive(value);
        }
    }
    deepCopy() {
        throw Error('deepCopy is only implemented in subclasses of Observable');
    }
    static isObservable(obj) {
        return obj && obj.isObservable;
    }
}
