import React, { useLayoutEffect, useState } from 'react';
import { render as r3fRender, advance as r3fAdvance, useFrame, extend, unmountComponentAtNode } from '@react-three/fiber';
import * as THREE from 'three';
import { initEvents } from './events';
import { MpSdkContext } from './hooks';
import { ObservableValue } from './ObservableValue';
export class R3FSDKAdapter {
    constructor(sdk) {
        this.r3fStore = null;
        this.r3fElements = new ObservableValue([]);
        this.mpScene = null;
        this.onReady = ({ camera, renderer, scene, registerMeshEvents }) => {
            var _a, _b;
            renderer.outputEncoding = THREE.sRGBEncoding;
            this.mpScene = scene;
            const r3fConfig = {
                gl: renderer,
                camera: camera,
                size: {
                    top: 0,
                    left: 0,
                    width: (_a = renderer.domElement.clientWidth) !== null && _a !== void 0 ? _a : window.innerWidth,
                    height: (_b = renderer.domElement.clientHeight) !== null && _b !== void 0 ? _b : window.innerHeight,
                },
                dpr: renderer.getPixelRatio(),
                frameloop: 'never',
            };
            this.r3fStore = r3fRender(React.createElement(R3FRoot, { r3fElements: this.r3fElements, sdk: this.sdk }), renderer.domElement, r3fConfig);
            const rootState = this.r3fStore.getState();
            initEvents(this.r3fStore, registerMeshEvents);
            this.r3fStore.getState().xr.disconnect();
            scene.add(rootState.scene);
        };
        this.onFrame = () => {
            if (this.r3fStore) {
                const rootState = this.r3fStore.getState();
                this.envSync(rootState.scene);
                r3fAdvance(performance.now() / 1000, true, rootState);
            }
        };
        this.onSizeChange = (width, height) => {
            var _a;
            (_a = this.r3fStore) === null || _a === void 0 ? void 0 : _a.getState().setSize(width, height);
        };
        this.onPixelRatioChange = (newRatio) => {
            var _a;
            (_a = this.r3fStore) === null || _a === void 0 ? void 0 : _a.getState().setDpr(newRatio);
        };
        this.envSync = (scene) => {
            var _a;
            if (this.mpScene && scene.environment && !((_a = this.mpScene) === null || _a === void 0 ? void 0 : _a.environment)) {
                this.mpScene.environment = scene.environment;
            }
        };
        this.sdk = sdk;
        sdk.R3F.registerR3F({
            onFrame: this.onFrame,
            onSizeChange: this.onSizeChange,
            onPixelRatioChange: this.onPixelRatioChange,
        }).then(this.onReady);
    }
    clear() {
        this.r3fElements.value = [];
    }
    dispose() {
        var _a, _b;
        this.clear();
        const rootState = (_a = this.r3fStore) === null || _a === void 0 ? void 0 : _a.getState();
        if (rootState) {
            if (rootState.scene)
                (_b = this.mpScene) === null || _b === void 0 ? void 0 : _b.remove(rootState.scene);
            if (rootState.gl) {
                unmountComponentAtNode(rootState.gl.domElement, (c) => {
                    console.log('disposed r3f scene', c);
                });
            }
        }
        this.mpScene = null;
        this.r3fStore = null;
    }
    register(element) {
        this.r3fElements.value = [...this.r3fElements.value, typeof element === 'function' ? element() : element];
    }
}
export function R3FRoot(props) {
    React.useMemo(() => extend(THREE), []);
    useFrame(() => { }, 1);
    const [elements, setElements] = useState(props.r3fElements.value);
    useLayoutEffect(() => {
        const { cancel } = props.r3fElements.onChanged(setElements);
        return cancel;
    }, [props.r3fElements]);
    return (React.createElement(MpSdkContext.Provider, { value: props.sdk }, elements === null || elements === void 0 ? void 0 : elements.map((element, i) => (React.createElement(ErrorBoundary, { key: i },
        React.createElement(React.Suspense, { fallback: null }, element))))));
}
class ErrorBoundary extends React.Component {
    constructor() {
        super(...arguments);
        this.state = { error: false };
    }
    componentDidCatch(error, errorInfo) {
        console.error(error, errorInfo);
    }
    render() {
        return this.state.error ? null : this.props.children;
    }
}
ErrorBoundary.getDerivedStateFromError = () => ({ error: true });
